//
//  ColorPicker.swift
//  ColorPickerTest
//
//  Created by Pierre Tacchi on 13/06/22.
//

import Cocoa

import os.log

var vx:NSView!

var s0:NSSlider!
var t0:NSTextField!

var s1:NSSlider!
var t1:NSTextField!

extension NSWindow {

    func toolbarHeight() -> CGFloat {
        var toolbar: NSToolbar?
        var toolbarHeight = CGFloat(0.0)
        var windowFrame: NSRect

        toolbar = self.toolbar

        if let toolbar = toolbar {
            if toolbar.isVisible {
                windowFrame = NSWindow.contentRect(forFrameRect: self.frame, styleMask: self.styleMask)
                toolbarHeight = NSHeight(windowFrame) - NSHeight(self.contentView!.frame)
            }
        }

        return toolbarHeight
    }
}

@objc public final class ColorPicker: NSColorPicker, NSColorPickingCustom {    
    @IBOutlet var view: NSView!
    @IBOutlet weak var label: NSTextField!
    
    public func supportsMode(_ mode: NSColorPanel.Mode) -> Bool {
        mode.rawValue == 123
    }
    
    public func currentMode() -> NSColorPanel.Mode {
        withUnsafePointer(to: Int(123)) {
            $0.withMemoryRebound(to: NSColorPanel.Mode.self, capacity: 1) {
                $0.pointee
            }
        }
    }
    
    public override var provideNewButtonImage: NSImage {
        guard
            let bundle = Bundle(identifier: "c.p.ColorPicker"),
            let buttonImage = bundle.image(forResource: "technical.difficulties")
        else { fatalError("Image Loading Error") }
        
        return buttonImage
    }
    
    public func provideNewView(_ initialRequest: Bool) -> NSView {
        if initialRequest {
            guard
                let bundle = Bundle(identifier: "c.p.ColorPicker"),
                bundle.loadNibNamed("Interface", owner: self, topLevelObjects: nil) else {
                fatalError("Can't load Fireworks nib.")
            }
            
            return view
        }
        return view ?? .init()
    }

    @objc func ChgSize()
    {
        let rc = view.bounds

        let tb = view.window?.toolbarHeight()

        s0.frame = NSMakeRect(5,rc.height - 31 - tb!,200,26)

        t0.frame = NSMakeRect(rc.width - 110,rc.height - 31 - tb!,100,26)
    }

    public func setColor(_ newColor: NSColor) {
 //       label.stringValue = newColor.description

        let rc = view.bounds

        let tb = view.window?.toolbarHeight()
        
        if vx == nil
        {
            vx = NSView.init(frame: rc)

            vx.window?.preservesContentDuringLiveResize = true
            
            vx.autoresizesSubviews = false
            
            vx.autoresizingMask = [.width,.height]
            
            s0 = NSSlider.init(frame: NSMakeRect(5,rc.height - 31 - tb!,200,26))
            t0 = NSTextField.init(frame: NSMakeRect(rc.width - 110,rc.height - 31 - tb!,100,26))

            s0.maxValue = 10
            
            s0.doubleValue = 5
            
            t0.stringValue = "255"
            
            s0.translatesAutoresizingMaskIntoConstraints = false
            t0.translatesAutoresizingMaskIntoConstraints = false

            s0.autoresizingMask = [.maxXMargin,.maxYMargin,.width]

            vx.postsFrameChangedNotifications = true

            NotificationCenter.default.addObserver(self, selector:#selector(ChgSize), name: NSView.frameDidChangeNotification, object: vx)
            t0.autoresizingMask = [.minXMargin,.maxYMargin]

                    vx.addSubview(s0)
                    vx.addSubview(t0)

                    view.addSubview(vx)
            
        }

    }
}
